@@ -1,4 +1,4 @@
-;; Nyacc eXtension for Tcl
+;;; language/nx-tcl/spec.scm - Nyacc eXtension for Tcl
 
 ;; Copyright (C) 2018 Matthew R. Wette
 ;;
@@ -25,9 +25,12 @@
 
 (define-language nx-tcl
   #:title	"nx-tcl"
-  #:reader	read-tcl-stmt
+  #:reader	(lambda (p e) (if (eq? e (interaction-environment))
+  				  (read-tcl-stmt p e)
+  				  (read-tcl-file p e)))
   #:compilers   `((tree-il . ,compile-tree-il))
   #:evaluator	(lambda (exp mod) (primitive-eval exp))
+  #:printer	write
   #:make-default-environment
 		(lambda ()
 		  (let ((env (make-fresh-user-module)))
