@@ -4,14 +4,13 @@
 (use-modules (srfi srfi-1))
 (use-modules (nyacc lang c99 parser))
 (use-modules (nyacc lang c99 cxeval))
-(use-modules (nyacc lang c99 xparser))
 (use-modules (nyacc lang c99 cpp))
 (use-modules (nyacc lang c99 pprint))
 (use-modules (nyacc lang c99 munge))
 (use-modules (nyacc lang c99 cxeval))
 (use-modules (nyacc lang c99 util1))
 (use-modules (nyacc lang util))
-(use-modules (nyacc lang sx-match))
+(use-modules (nyacc lang sx-util))
 (use-modules (nyacc lex))
 (use-modules (nyacc util))
 (use-modules (ice-9 pretty-print))
