@@ -34,6 +34,29 @@
 ;; done.  I'm still absorbing those and don't know how much insight I can
 ;; provide. -- Matt - 15 Jul 2018
 
+;;; Notes:
+
+;; @itemize
+;; @item JS functions will need to be re-implemented as objects, with the
+;;       `[[Call]]' property used to make calls.
+;; @item don't support the arguments property in functions
+;; @item could maybe handle this using
+;;  d.f1 = function... => d.f1 = (let ((this d)) f1)
+;;  but then d1.f = d0.f does not work
+;; @item (void) == (const *unspecified*) i think
+;; @item need atomic-box?
+;; @end itemize
+
+;; https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/\
+;;   Details_of_the_Object_Model
+;; If Manager: Employee
+;; function Manager() {
+;;   Employee.call(this);
+;;   this.reports = [];
+;; }
+;; Manager.prototype = Object.create(Employee.prototype);
+;; Manager.prototype.constructor = Manager;
+
 ;; TODO:
 ;; @itemize
 ;; @item Imeplement @code{with}.
@@ -49,23 +72,10 @@
 ;; @item add let only allow var at top-level and function start scope
 ;; @end itemize
 
-;;; Notes:
-
-;; @itemize
-;; @item JS functions will need to be re-implemented as objects, with the
-;;       `[[Call]]' property used to make calls.
-;; @item don't support the arguments property in functions
-;; @item could maybe handle this using
-;;  d.f1 = function... => d.f1 = (let ((this d)) f1)
-;;  but then d1.f = d0.f does not work
-;; @item (void) == (const *unspecified*) i think
-;; @item need atomic-box?
-;; @end itemize
-
 ;;; Code:
 
 (define-module (nyacc lang javascript compile-tree-il)
-  #:export (compile-tree-il)
+  #:export (compile-tree-il show-javascript-sxml show-javascript-xtil)
   #:use-module (nyacc lang javascript xlib)
   #:use-module (nyacc lang sx-util)
   #:use-module (nyacc lang nx-util)
@@ -187,10 +197,10 @@
 ;; @end example
 ;; @end deffn
 (define (make-let bindings exprs)
-  (let iter ((names '()) (gsyms '()) (vals '()) (bindings bindings))
+  (let loop ((names '()) (gsyms '()) (vals '()) (bindings bindings))
     (if (null? bindings)
 	`(let ,(reverse names) ,(reverse gsyms) ,(reverse vals) ,(block exprs))
-	(iter (cons (list-ref (car bindings) 1) names)
+	(loop (cons (list-ref (car bindings) 1) names)
 	      (cons (list-ref (car bindings) 2) gsyms)
 	      (cons (list-ref (car bindings) 3) vals)
 	      (cdr bindings)))))
@@ -206,35 +216,41 @@
 ;; @end example
 ;; @noindent
 ;; Oh, I think this needs to use @code{letrec*}.
+;; @end deffn
 (define (wrap-bindings body)
   ;;(sferr "wrap:\n  body\n") (pperr body)
-  (let iter1 ((bindings '()) (rest body))
+  (let loop1 ((bindings '()) (rest body))
     (match rest
       (`(seq (bindings . ,bnds) ,rst)
-       (iter1 (fold cons bindings bnds) rst))
+       (loop1 (fold cons bindings bnds) rst))
       (_
        ;;(sferr "  bindings, rest:\n") (pperr bindings) (pperr rest)
        (if (null? bindings)
 	   body
-	   (let iter2 ((names '()) (gsyms '()) (inits '()) (binds bindings))
+	   (let loop2 ((names '()) (gsyms '()) (inits '()) (binds bindings))
 	     (if (null? binds)
 		 `(letrec* ,names ,gsyms ,inits ,rest)
-		 (iter2 (cons (list-ref (car binds) 1) names)
+		 (loop2 (cons (list-ref (car binds) 1) names)
 			(cons (list-ref (car binds) 2) gsyms)
 			(cons (list-ref (car binds) 3) inits)
 			(cdr binds)))))))))
 
-;; Pass arguments as optional and add @code{this} keyword argument.
+;; @deffn {Procedure} make-function name this args body
+;; Pass arguments as optional and add @code{this} keyword argument.@*
+;; Note: Change this @code{make-arity} (see tcl/c-t-il.scm).
+;; @end deffn.
 (define (make-function name this args body)
-  (if (not args) (error "no args"))
-  `(lambda ,(if name `((name . ,name)) '())
-     (lambda-case ((()			; req
-		    ,(map car args)	; opt
-		    #f			; rest
-		    (#f (#:this this ,this)) ; kw
-		    (,@(map (lambda (v) undefined) args) ,undefined) ; inits
-		    (,@(map cadr args) ,this)) ; syms
-		   ,body))))
+  (unless args (error "no args"))
+  (let* ((meta '((language . nx-javascript)))
+	 (meta (if name (cons `(name . ,name) meta) meta)))
+    `(lambda ,meta
+       (lambda-case ((()			; req
+		      ,(map car args)	; opt
+		      #f			; rest
+		      (#f (#:this this ,this)) ; kw
+		      (,@(map (lambda (v) undefined) args) ,undefined) ; inits
+		      (,@(map cadr args) ,this)) ; syms
+		     ,body)))))
 
 ;; used in fU for switch cases
 ;; (let ((key (if key #t (equal? case-key val))))
@@ -252,7 +268,8 @@
 	    (prim-call equal? ,val ,(cadr kseed))))
 	(seq (if (lexical ~key ,sym) ,(car kseed) (void))
 	       ,next)))
-	 
+
+;; NOT USED -- NOT NEEDED
 ;; @deffn {Procedure} resolve-ref ref => exp
 ;; WARNING: I think this is more subtle than I am making it.@*
 ;; Resolve a possible reference (lval) to an expression (rval).
@@ -291,9 +308,9 @@
 (define (op-call/prim op kseed)
   (rev/repl 'prim-call op kseed))
 
-;; deffn {Procedure} op-assn kseed => `(set! lhs rhs)
+;; @deffn {Procedure} op-assn kseed => `(set! lhs rhs)
 ;; op-assn: for lhs += rhs etc
-;; end deffn
+;; @end deffn
 (define op-assn
   (let ((opmap
 	 '((mul-assign . js:*) (div-assign . js:/) (mod-assign . js:%)
@@ -329,12 +346,12 @@
   ;; @deffn {Procedure} remove-empties elements => elements
   ;; @end deffn
   (define (remove-empties elements)
-    (let iter ((elts elements))
+    (let loop ((elts elements))
       (if (null? elts) '()
 	  (let ((elt (car elts)) (rest (cdr elts)))
 	    (if (eq? (car elt) 'EmptyStatement)
-		(iter rest)
-		(cons elt (iter rest)))))))
+		(loop rest)
+		(cons elt (loop rest)))))))
 
   ;; @deffn {Procedure} labelable-stmt? stmt => #f|stmt
   ;; This predicate determines if the statement can have a preceeding label.
@@ -356,7 +373,7 @@
   ;; @code{switch}, or removes them if not preceeding iteration statement.
   ;; @end deffn
   (define (cleanup-labels src-elts-tail)
-    (let iter ((src src-elts-tail))
+    (let loop ((src src-elts-tail))
       (if (null? src) '()
 	  (if (eq? (caar src) 'LabelledStatement)
 	      (call-with-values
@@ -376,9 +393,9 @@
 			(simple-format (current-error-port)
 				       "removing misplaced label: ~A\n"
 				       (cadr id))
-			(iter rest))
-		      (cons `(LabelledStatement ,id ,stmt) (iter rest)))))
-	      (cons (car src) (iter (cdr src)))))))
+			(loop rest))
+		      (cons `(LabelledStatement ,id ,stmt) (loop rest)))))
+	      (cons (car src) (loop (cdr src)))))))
 
   ;; @deffn {Procedure} hoist-decls elements => elements
   ;; Move all variable declarations to front and replace old ones with
@@ -388,13 +405,13 @@
   (define (split-dstmt dstmt hoisted rest) ;; => hoisted rest
     ;; move Let/VariableStatement to hoisted, moving initializers
     ;; to decls as assignment statements
-    (let iter ((hdecls '()) (rest rest) (decls (cdadr dstmt)))
+    (let loop ((hdecls '()) (rest rest) (decls (cdadr dstmt)))
       (if (null? decls)
 	  (values (cons `(,(car dstmt) (DeclarationList ,hdecls)) hoisted) rest)
 	  (let* ((decl (car decls))
 		 (init (if (= 3 (length decl)) (caddr decl) #f))
 		 (decl (if init (list (car decl) (cadr decl)) decl)))
-	    (iter (cons decl hdecls)
+	    (loop (cons decl hdecls)
 		  (if init
 		      (cons `(ExpressionStatement
 			      (AssignmentExpression
@@ -404,26 +421,26 @@
 		      rest)
 		  (cdr decls))))))
   (define (hoist-decls elements)
-    (let ((tail (let iter ((elts elements)) ; tail after Let/VarStatements
+    (let ((tail (let loop ((elts elements)) ; tail after Let/VarStatements
 		  (cond
 		   ((null? elts) elts)
 		   ((not (memq (caar elts) hoist-stmts)) elts)
-		   (iter (cdr elts))))))
-      (let iter ((hoisted '()) (rest '()) (elts tail))
+		   (loop (cdr elts))))))
+      (let loop ((hoisted '()) (rest '()) (elts tail))
 	(cond
 	 ((null? elts)
 	  (if (null? hoisted)
 	      elements			   ; nothing hoisted
-	      (let iter2 ((elts elements)) ; rebuild statement list
+	      (let loop2 ((elts elements)) ; rebuild statement list
 		(if (eq? elts tail) (append-reverse hoisted (reverse rest))
-		    (cons (car elts) (iter2 (cdr elts)))))))
+		    (cons (car elts) (loop2 (cdr elts)))))))
 	 ((memq (caar elts) hoist-stmts)
 	  (call-with-values		; decl=>hoisted init=>rest
 	      (lambda () (split-dstmt (car elts) hoisted rest))
 	    (lambda (hoisted rest)
-	      (iter hoisted rest (cdr elts)))))
+	      (loop hoisted rest (cdr elts)))))
 	 (else
-	  (iter hoisted (cons (car elts) rest) (cdr elts)))))))
+	  (loop hoisted (cons (car elts) rest) (cdr elts)))))))
 	  
   
   ;; @deffn {Procedure} fold-in-blocks elts-tail => elts-tail
@@ -441,12 +458,12 @@
   ;; We assume no elements of @code{FunctionElements} is text.
   ;; @end deffn
   (define (x-fold-in-blocks src-elts-tail)
-    (let iter ((src  src-elts-tail))
+    (let loop ((src  src-elts-tail))
       (if (null? src) '()
 	  (let ((elt (car src)) (rest (cdr src)))
 	    (if (eq? (car elt) 'VariableStatement)
-		(list (cons* 'Block (cadr elt) (iter rest)))
-		(cons elt (iter rest)))))))
+		(list (cons* 'Block (cadr elt) (loop rest)))
+		(cons elt (loop rest)))))))
 
   ;; @deffn {Procedure} check-scoping elements
   ;; Check blocks to make sure @code{var} is only used in blocks at entry
@@ -494,16 +511,44 @@
       ((PrimaryExpression (StringLiteral ,str))
        (values '() `(const ,str) dict))
 
+      ((CallExpression
+	(obj-ref ,expr (PrimaryExpression (Identifier ,name))) ,arg-list)
+       (sferr "1\n")
+       (values `(obj-CallExpression ,expr ,name ,arg-list) '() dict))
+      ((CallExpression (obj-ref ,expr (Identifier ,name)) ,arg-list)
+       (sferr "2\n")
+       (values `(obj-CallExpression ,expr ,name ,arg-list) '() dict))
+      ((CallExpression . ,rest)
+       (values tree '() dict))
+
       ((PropertyNameAndValue (Identifier ,name) ,expr)
        (values `(PropertyNameAndValue (PropertyName ,name) ,expr) '() dict))
 
       ((obj-ref ,expr (Identifier ,name))
-       (values `(ooa-ref ,expr (PropertyName ,name)) '() dict))
+       (values `(ooa-ref ,expr ,name) '() dict))
 
       ((Block . ,elts) ;; see comments on FunctionElements below
        (let* ((elts (remove-empties elts))
 	      (elts (cleanup-labels elts)))
 	 (values tree '() dict)))
+
+      ;; Convert AssignmentExpression to:
+      ;; 1) var-AssignmentExpression => ... (set! ...)
+      ;; 2) obj-AssignmentExpression => ... (hash-set! ...)
+      ;; 3) ooa-AssignmentExpression => ... (vector-set! ...)|(hash-set! ...)
+      ;; TODO: check LeftHandSideExpression
+      ((AssignmentExpression
+	(@ . ,attr) (PrimaryExpression (Identifier ,name)) ,assn ,rhs)
+       (values `(var-AssignmentExpression ,(cdr tree)) '() dict))
+      ((AssignmentExpression
+	(@ . ,attr) (obj-ref ,expr (Identifier ,name)) ,assn ,rhs)
+       (values
+	`(obj-AssignmentExpression (@ . ,attr) ,expr ,name ,assn ,rhs)
+	'() dict))
+      ((AssignmentExpression (@ . ,attr) (ooa-ref ,expr ,expr) ,rhs)
+       (values
+	`(ooa-AssignmentExpression (@ . ,attr) ,expr ,expr ,rhs)
+	'() dict))
       
       ((StatementList . ,stmts)
        (let* ((stmts (remove-empties stmts))
@@ -554,7 +599,10 @@
 			     (push-scope (add-symbol "*anon*" dict)))))
       
       ((FormalParameterList . ,idlist)
-       (values tree '() (fold add-lexical dict (map cadr idlist))))
+       (values tree '()
+	       (acons 'arguments-used? #f
+		      (add-lexical "arguments"
+				   (fold add-lexical dict (map cadr idlist))))))
       
       ((FunctionElements . ,elts)
        ;; Fix up list of function elements:
@@ -596,7 +644,7 @@
        ;; PrimaryExpression (w/ ArrayLiteral or ObjectLiteral only)
        ((PrimaryExpression)
 	(values (cons (car kseed) seed) kdict))
-       
+
        ;; ArrayLiteral
        ;; mkary is just primitive vector
        ((ArrayLiteral)
@@ -615,10 +663,16 @@
 
        ;; ObjectLiteral
        ((ObjectLiteral)
-	(values (cons (car kseed) seed) kdict))
+	(let* ((tail (rtail kseed)))
+	  (values
+	   (cons
+	    (if (null? tail) `(call (toplevel make-hash-table)) (car tail))
+	    seed)
+	   kdict)))
        
        ;; PropertyNameAndValueList
        ((PropertyNameAndValueList)
+	(sferr "prop list\n") (pperr (reverse kseed))
 	(values
 	 (cons `(call (@@ ,xlib-mod mkobj) ,@(rtail kseed)) seed)
 	 kdict))
@@ -629,25 +683,45 @@
 
        ;; PropertyName
        ((PropertyName)
-	(values (cons `(const ,(car kseed)) seed) kdict))
+	(values (cons `(const ,(string->symbol (car kseed))) seed) kdict))
 
        ;; ooa-ref (object-or-array ref), a cons cell: (dict name)
        ;; obj-ref: converted to ooa-ref in fD
        ;; => (cons <expr> <name>)
        ;; a bit ugly now ???
        ((ooa-ref)
-	(values
-	 (cons `(prim-call cons
-			   (call ,(xlib-ref 'js-resolve) ,(cadr kseed))
-			   ,(car kseed))
-	       seed) kdict))
+	(sferr "ooaref:\n") (pperr (rtail kseed))
+	(let* ((expr `(primcall cons
+				 (call ,(xlib-ref 'js-resolve) ,(cadr kseed))
+				 ,(car kseed)))
+	       (tail (rtail kseed))
+	       (ooa (list-ref tail 0))
+	       (arg `(const ,(string->symbol (list-ref tail 1))))
+	       (expr `(call ,(xlib-ref 'js:ooa-ref) ,ooa ,arg))
+	       )
+	  (pperr expr)
+	  (values (cons expr seed) kdict)))
 
-       ;; new: for now just call object
-       ((new) (values (cons (car kseed) seed) kdict))
+       ;; new
+       ((new)
+	(let* ((tail (rtail kseed))
+	       (expr `(call ,(car tail) ,@(cadr tail) (const #:this)
+			    (call (toplevel make-hash-table)))))
+	  (values (cons expr seed) kdict)))
        
-       ;; CallExpression ;; this should probably insert a (de-ref fct)(args)
-       ;; Should defined be boxed?
-       ((CallExpression) ;; need to deal with "this"
+        ;; obj-CallExpression obj name args : add args #:this obj
+       ((obj-CallExpression)
+	(let* ((tail (rtail kseed))
+	       (obj (list-ref tail 0))
+	       (mem `(const ,(string->symbol (list-ref tail 1))))
+	       (args (list-ref tail 2))
+	       (args (append args (list `(const #:this) obj)))
+	       (expr `(call (call ,(xlib-ref 'js:ooa-ref) ,obj ,mem) . ,args)))
+	  ;;(sferr "obj-C\n") (pperr tail)
+	  (values (cons expr seed) kdict)))
+       
+       ;; CallExpression
+       ((CallExpression)
 	;;(pperr (cons* 'apply (cadr kseed) (car kseed)))
 	(values (cons (cons* 'call (cadr kseed) (car kseed)) seed) kdict))
 
@@ -726,9 +800,20 @@
        ;; assign mul-assign div-assign od-assign add-assign sub-assign
        ;; lshift-assign rshift-assign rrshift-assign and-assign
        ;; xor-assign or-assign
-       ((AssignmentExpression)
-	(values (cons (op-assn kseed) seed) kdict))
+       ;; Note that assignment needs to return the value always
+       ((var-AssignmentExpression)
+	(let* ((tail (rtail kseed))
+	       )
+	  (sferr "obj-Ass\n") (pperr tail)
+	  (values (cons (op-assn kseed) seed) kdict)))
 
+       ((obj-AssignmentExpression)
+	(let* ((tail (rtail kseed))
+	       )
+	  (sferr "obj-Ass\n") (pperr tail)
+	  (values (cons '(void) seed) dict)))
+	;;;;`(obj-AssignmentExpression (@ . ,attr) ,expr ,name ,op ,rhs)
+       
        ;; expr-list
 
        ;; Block : has same elements as StatementList
@@ -897,11 +982,11 @@
 	(values
 	 (cons 
 	  (let ((val (lookup "swx~val" kdict)))
-	    (let iter ((next '(void)) (sym (jsym)) (ks kseed))
+	    (let loop ((next '(void)) (sym (jsym)) (ks kseed))
 	      (if (eq? (car ks) 'CaseClauses)
 		  `(let (~key) (,sym) ((const #f)) ,next)
 		  (let ((psym (jsym)))
-		    (iter (make-case val sym psym (car ks) next)
+		    (loop (make-case val sym psym (car ks) next)
 			  psym (cdr ks))))))
 	  seed)
 	 kdict))
@@ -978,6 +1063,7 @@
        ;; FunctionDeclaration (see also fD above)
        ;; If the body starts with (bindings ...) then make a let.
        ((FunctionDeclaration)
+	;; TODO: check for arguments-used?
 	(let* ((name (let ((n (caddr kseed)))
 		       (if (memq (car n) '(@ @@)) (caddr n) (cadr n))))
 	       (this (caddr (lookup "this" kdict)))
@@ -986,6 +1072,8 @@
 	       (body (wrap-bindings (car kseed)))
 	       (body (with-escape/arg ptag (block body)))
 	       (fctn (make-function name this args body)))
+	  (if (assq-ref kdict 'arguments-used?)
+	      (throw 'javascript-error "function `arguments' not supported yet"))
 	  (values (cons `(define ,name ,fctn) seed) (pop-scope kdict))))
 
        ;; FunctionExpression
@@ -1001,6 +1089,8 @@
 	       (body (with-escape/arg ptag (block body)))
 	       (fctn (make-function name this args body))
 	       (fctn (if lref `(let (,name) (,gsym) (,fctn) ,lref))))
+	  (if (assq-ref kdict 'arguments-used?)
+	      (throw 'javascript-error "function `arguments' not supported yet"))
 	  (values (cons fctn seed) (pop-scope kdict))))
 
        ;; FormalParameterList
@@ -1033,18 +1123,21 @@
 
 (use-modules (language tree-il compile-cps))
 
+(define show-sxml #f)
+(define (show-javascript-sxml v) (set! show-sxml v))
+(define show-xtil #f)
+(define (show-javascript-xtil v) (set! show-xtil v))
+
 (define (compile-tree-il exp env opts)
-  ;;(sferr "\nenv=~S\n" env)
-  ;;(sferr "sxml:\n") (pperr exp)
+  (when show-sxml (sferr "sxml:\n") (pperr exp))
   (let ((cenv (if (module? env) (acons '@top #t (acons '@M env JSdict)) env)))
-    ;;(sferr "env=~S\ncenv=~S" env cenv)
     (if exp 
 	(call-with-values
 	    (lambda () (xlang-sxml->xtil exp cenv opts))
 	  (lambda (exp cenv)
-	    ;;(sferr "tree-il:\n") (pperr exp)
+	    (when show-xtil (sferr "tree-il:\n") (pperr exp))
 	    (values (parse-tree-il exp) env cenv)
-	    ;;(values (parse-tree-il '(const "[compile-tree-il skip]")) env cenv)
+	    ;;(values (parse-tree-il '(const "[hello]")) env cenv)
      	    ))
 	(values (parse-tree-il '(void)) env cenv))))
 
