@@ -15,7 +15,9 @@
 ;; You should have received a copy of the GNU Lesser General Public License
 ;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
-;; JavaScript parser
+;;; Description:
+
+;; This is a JavaScript parser for Guile.
 
 (define-module (nyacc lang javascript parser)
   #:export (parse-js js-stmt-reader js-file-reader)
@@ -28,16 +30,13 @@
 
 ;; === file parser ===================
 
-(include-from-path "nyacc/lang/javascript/mach.d/jstab.scm")
-(include-from-path "nyacc/lang/javascript/mach.d/jsact.scm")
+(include-from-path "nyacc/lang/javascript/mach.d/js-tab.scm")
+(include-from-path "nyacc/lang/javascript/mach.d/js-act.scm")
 
 (define gen-js-lexer (make-js-lexer-generator js-mtab))
 
 (define raw-parser
-  (make-lalr-parser 
-   (list (cons 'len-v js-len-v) (cons 'pat-v js-pat-v)
-    (cons 'rto-v js-rto-v) (cons 'mtab js-mtab)
-    (cons 'act-v js-act-v))))
+  (make-lalr-parser (acons 'act-v js-act-v js-tables)))
 
 ;; @deffn {Procedure} parse-js-file [#:debug bool] 
 ;; to be documented
@@ -55,23 +54,21 @@
 ;; Read file unit from port and return SXML AST.
 ;; @end deffn
 (define (js-file-reader port env)
-  (with-input-from-port port parse-js))
+  (if (eof-object? (peek-char port))
+      (read-char port)
+      (with-input-from-port port parse-js)))
 
 ;; === interactive parser =============
 
-(include-from-path "nyacc/lang/javascript/mach.d/ia-jstab.scm")
-(include-from-path "nyacc/lang/javascript/mach.d/ia-jsact.scm")
+(include-from-path "nyacc/lang/javascript/mach.d/ia-js-tab.scm")
+(include-from-path "nyacc/lang/javascript/mach.d/ia-js-act.scm")
 
 (define gen-ia-js-lexer (make-js-lexer-generator ia-js-mtab))
 
 (define raw-ia-parser
-  ;;(make-lalr-ia-parser/num		; TEMPORARY 
   (make-lalr-parser
-   (list (cons 'len-v ia-js-len-v) (cons 'pat-v ia-js-pat-v)
-	 (cons 'rto-v ia-js-rto-v) (cons 'mtab ia-js-mtab)
-	 (cons 'act-v ia-js-act-v))
-   #:interactive #t
-   ))
+   (acons 'act-v ia-js-act-v ia-js-tables)
+   #:interactive #t))
 
 ;; @deffn {Procedure} parse-js-stmt
 ;; Parse a program statement from interactive user.
@@ -90,12 +87,12 @@
 (define flush-input-after-error
   (let ((read-string (make-string-reader #\")))
     (lambda (port)
-      (let iter ((ch (read-char port)))
+      (let loop ((ch (read-char port)))
 	(cond
 	 ((eqv? ch #\;) #f)
-	 ((read-js-string ch) (iter (read-char port)))
+	 ((read-js-string ch) (loop (read-char port)))
 	 ((read-c-comm ch #t) #f)
-	 (else (iter (read-char port))))))))
+	 (else (loop (read-char port))))))))
 
 (define (js-stmt-reader port env)
   (if (eof-object? (peek-char port))
