@@ -963,4 +963,11 @@
     ("++" . 98) ("->" . 99) ("." . 100) ("]" . 101) ("[" . 102) ("}" . 103) 
     ("{" . 104) (")" . 105) ("(" . 106) ($error . 2) ($end . 108)))
 
+(define c99x-tables
+  (list
+   (cons 'len-v c99x-len-v)
+   (cons 'pat-v c99x-pat-v)
+   (cons 'rto-v c99x-rto-v)
+   (cons 'mtab c99x-mtab)))
+
 ;;; end tables
