@@ -1,5 +1,5 @@
-;; nyacc/lex.scm
-;;
+;;; nyacc/lex.scm
+
 ;; Copyright (C) 2015-2017 - Matthew R.Wette
 ;; 
 ;; This library is free software; you can redistribute it and/or modify it
@@ -15,6 +15,8 @@
 ;; You should have received a copy of the GNU Lesser General Public License
 ;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
+;;; Description:
+
 ;; A module providing procedures for constructing lexical analyzers.
 
 ;; '$fixed '$float '$string '$ident '$chlit '$chlit/L '$chlit/u '$chlit/U
@@ -31,6 +33,8 @@
 ;; 
 ;; read-c-ident 
 
+;;; Code:
+
 (define-module (nyacc lex)
   #:export (make-lexer-generator
 	    make-ident-reader
@@ -329,7 +333,7 @@
 (define (make-num-reader)
   ;; This will incorrectly parse 123LUL.  Does not handler hex floats.
   ;; 0: start; 1: p-i; 2: p-f; 3: p-e-sign; 4: p-e-d; 5: packup
-  ;; Removed support for leading '.' to be a number.
+  ;; NO LONGER Removed support for leading '.' to be a number.
   (lambda (ch1)
     ;; chl: char list; ty: '$fixed or '$float; st: state; ch: next ch; ba: base
     (let iter ((chl '()) (ty #f) (ba 10) (st 0) (ch ch1))
@@ -339,12 +343,18 @@
 	  ((eof-object? ch) (iter chl ty ba 5 ch))
 	  ((char=? #\0 ch) (iter (cons ch chl) '$fixed 8 10 (read-char))) 
 	  ((char-numeric? ch) (iter chl '$fixed ba 1 ch))
+	  ((char=? #\. ch) (iter (cons ch chl) #f ba 15 (read-char))) 
 	  (else #f)))
 	((10) ;; allow x after 0
 	 (cond
 	  ((eof-object? ch) (iter chl ty ba 5 ch))
 	  ((char=? #\x ch) (iter (cons ch chl) ty 16 1 (read-char)))
 	  (else (iter chl ty ba 1 ch))))
+	((15) ;; got `.' only
+	 (cond
+	  ((eof-object? ch) (unread-char ch) #f)
+	  ((char-numeric? ch) (iter (cons ch chl) '$float ba 2 (read-char)))
+	  (else (unread-char ch) #f)))
 	((1)
 	 (cond
 	  ((eof-object? ch) (iter chl ty ba 5 ch))
