@@ -1,18 +1,18 @@
-;; nyacc/parse.scm
+;;; nyacc/parse.scm
 
 ;; Copyright (C) 2014-2018 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
-; modify it under the terms of the GNU Lesser General Public
+;; modify it under the terms of the GNU Lesser General Public
 ;; License as published by the Free Software Foundation; either
 ;; version 3 of the License, or (at your option) any later version.
 ;;
 ;; This library is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ;; Lesser General Public License for more details.
 ;;
-;;; You should have received a copy of the GNU Lesser General Public License
+;; You should have received a copy of the GNU Lesser General Public License
 ;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
 ;;; Description:
@@ -59,7 +59,7 @@
 (define (dmsg/n s t a)
   (cond
    ((not a) (sferr "state ~S, token ~S\t=> parse error\n" s t))
-   ((positive? a) (sferr "state ~S, token ~S\t=> shift ~S\n" s t a))
+   ((positive? a) (sferr "state ~S, token ~S\t=> shift, goto ~S\n" s t a))
    ((negative? a) (sferr "state ~S, token ~S\t=> reduce ~S\n" s t (- a)))
    ((zero? a) (sferr "state ~S, token ~S\t=> accept\n" s t))
    (else (error "coding error in (nyacc parse)"))))
@@ -67,29 +67,18 @@
 (define (dmsg/s s t a)
   (case (car a)
     ((error) (sferr "state ~S, token ~S\t=> parse error\n" s t))
-    ((shift) (sferr "state ~S, token ~S\t=> shift ~S\n" s t (cdr a)))
+    ((shift) (sferr "state ~S, token ~S\t=> shift, goto ~S\n" s t (cdr a)))
     ((reduce) (sferr "state ~S, token ~S\t=> reduce ~S\n" s t (cdr a)))
     ((accept) (sferr "state ~S, token ~S\t=> accept\n" s t))
     (else (error "coding error in (nyacc parse)"))))
 
 (define (parse-error state laval)
   (let ((fn (or (port-filename (current-input-port)) "(unknown)"))
-	(ln (port-line (current-input-port))))
+	(ln (1+ (port-line (current-input-port)))))
     (throw 'nyacc-error
 	   "~A:~A: parse failed at state ~A, on input ~S\n"
 	   fn ln (car state) (cdr laval))))
 
-;; @item make-lalr-parser mach [#:skip-if-unexp '()] [#:interactive #f] => parser
-;; This generates a procedure that takes one argument, a lexical analyzer:
-;; @example
-;; (parser lexical-analyzer [#:debug #t])
-;; @end example
-;; and is used as
-;; @example
-;; (define xyz-parse (make-lalr-parser xyz-mach))
-;; (with-input-from-file "sourcefile.xyz" (lambda () (xyz-parse (gen-lexer))))
-;; @end example
-;; The generated parser is reentrant.
 (define* (make-lalr-parser/sym mach #:key (skip-if-unexp '()) interactive)
   (let* ((len-v (assq-ref mach 'len-v))
 	 (rto-v (assq-ref mach 'rto-v))
@@ -184,7 +173,51 @@
 	     (else			; accept
 	      (car stack))))))))))
 
+;; @deffn {Procedure} make-lalr-parser mach [options] => parser
+;; Generate a procedure for parsing a language, where @var{mach} is
+;; a machine generated by @code{make-lalr-machine}.
+;; This generates a procedure that takes one argument, a lexical analyzer:
+;; @example
+;; (parser lexical-analyzer #:debug #t)
+;; @end example
+;; @noindent
+;; and is used as
+;; @example
+;; (define xyz-parse (make-lalr-parser xyz-mach))
+;; (with-input-from-file "sourcefile.xyz"
+;;   (lambda () (xyz-parse (gen-lexer))))
+;; @end example
+;; @noindent
+;; The generated parser is reentrant.  Options are:
+;; @table @code
+;; @item #:skip-if-unexp 
+;; This is a list of tokens to skip if not expected.  It is used
+;; to allow comments to be skipped.  The default is @code{'()}.
+;; @item #:interactive
+;; If @code{#t}, this tells the parserthat this is being called
+;; interactively, so that the token @code{$end} is not expected.
+;; The default value is @code{#f}.
+;; @end table
+;; @noindent
+;; @end deffn
 (define* (make-lalr-parser mach #:key (skip-if-unexp '()) interactive)
+  "- Procedure: make-lalr-parser mach [options] => parser
+     Generate a procedure for parsing a language, where MACH is a
+     machine generated by 'make-lalr-machine'.  This generates a
+     procedure that takes one argument, a lexical analyzer:
+          (parser lexical-analyzer #:debug #t)
+     and is used as
+          (define xyz-parse (make-lalr-parser xyz-mach))
+          (with-input-from-file \"sourcefile.xyz\"
+            (lambda () (xyz-parse (gen-lexer))))
+     The generated parser is reentrant.  Options are:
+     '#:skip-if-unexp'
+          This is a list of tokens to skip if not expected.  It is used
+          to allow comments to be skipped.  The default is ''()'.
+     '#:interactive'
+          If '#t', this tells the parserthat this is being called
+          interactively, so that the token '$end' is not expected.  The
+          default value is '#f'."
   (let* ((mtab (assq-ref mach 'mtab))
 	 (siu (map (lambda (n) (assoc-ref mtab n)) skip-if-unexp))
 	 (iact interactive))
@@ -274,116 +307,4 @@
 	      (else ;; accept
 	       (car stack))))))))))
 
-;; @deffn {Procedure} make-lalr-ia-parser mach [#:skip-if-unexp ()]
-;; Make an interactive parser.  The machine should have been built with
-;; @code{(compact-machine mach #:keep 0)} so that default reductions can
-;; be processed without grabbing a lookahead token.  If the optional
-;; keyword @var{#:skip-if-unexp} is provided then that list of tokens
-;; (e.g., for comments) will be skipped if the machine does not expect
-;; them.  If you add non-terminals to @var{skip-if-unexp} then you need
-;; to add those to @code{#:keepers} for @code{compact-machine}
-;; @end deffn
-
-(define* (make-lalr-ia-parser/sym mach #:key (skip-if-unexp '()))
-  (let* ((mtab (assq-ref mach 'mtab))
-	 (len-v (assq-ref mach 'len-v))
-	 (rto-v (assq-ref mach 'rto-v))
-	 (pat-v (assq-ref mach 'pat-v))
-	 (xct-v (make-xct (assq-ref mach 'act-v)))
-	 (start (assq-ref (assq-ref mach 'mtab) '$start)))
-    (lambda* (lexr #:key debug)
-      (let iter ((state (list 0))	; state stack
-		 (stack (list '$@))	; sval stack
-		 (nval #f)		; prev reduce to non-term val
-		 (lval #f))		; lexical value (from lex'er)
-	(cond
-	 ((and nval (eqv? (car nval) start)) ; done
-	  (cdr nval))
-	 ((not (or nval lval))
-	  (if (eqv? '$default (caar (vector-ref pat-v (car state))))
-	      (iter state stack (cons '$default #f) lval) ; default reduction
-	      (iter state stack nval (lexr))))		  ; reload
-	 (else
-	  (let* ((laval (or nval lval))
-		 (tval (car laval))
-		 (sval (cdr laval))
-		 (stxl (vector-ref pat-v (car state)))
-		 (stx (or (assq-ref stxl tval)
-			  (assq-ref stxl '$default)
-			  #f)))
-	    (if debug (dmsg/n (car state) (if nval tval sval) stx))
-	    (cond
-	     ((eq? '$error (car stx))	; error ???
-	      (if (memq tval skip-if-unexp)
-		  (iter state stack #f #f)
-		  (parse-error state laval)))
-	     ((eq? 'reduce (car stx))	; reduce
-	      (let* ((gx (cdr stx))
-		     (gl (vector-ref len-v gx))
-		     ($$ (apply (vector-ref xct-v gx) stack)))
-		(iter (list-tail state gl)
-		      (list-tail stack gl)
-		      (cons (vector-ref rto-v gx) $$)
-		      lval)))
-	     ((eq? 'shift (car stx))	; shift
-	      (iter (cons (cdr stx) state) (cons sval stack)
-		    #f (if nval lval #f)))
-	     (else			; accept
-	      (car stack))))))))))
-
-(define* (make-lalr-ia-parser/num mach #:key (skip-if-unexp '()))
-  (let* ((len-v (assq-ref mach 'len-v))
-	 (rto-v (assq-ref mach 'rto-v))
-	 (pat-v (assq-ref mach 'pat-v))
-	 (xct-v (make-xct (assq-ref mach 'act-v)))
-	 (start (assq-ref (assq-ref mach 'mtab) '$start)))
-    (lambda* (lexr #:key debug)
-      (let iter ((state (list 0))	; state stack
-		 (stack (list '$@))	; sval stack
-		 (nval #f)		; prev reduce to non-term val
-		 (lval #f))		; lexical value (from lex'er)
-	(cond
-	 ((and nval (eqv? (car nval) start)) ; done
-	  (cdr nval))
-	 ((not (or nval lval))
-	  (if (eqv? $default (caar (vector-ref pat-v (car state))))
-	      (iter state stack (cons $default #f) lval) ; default reduction
-	      (iter state stack nval (lexr))))		 ; reload
-	 (else
-	  (let* ((laval (or nval lval))
-		 (tval (car laval))
-		 (sval (cdr laval))
-		 (stxl (vector-ref pat-v (car state)))
-		 (stx (or (assq-ref stxl tval)
-			  (assq-ref stxl '$default)
-			  #f)))
-	    (if debug (dmsg/n (car state) (if nval tval sval) stx))
-	    (cond
-	     ((eq? #f stx)		; error
-	      (if (memq tval skip-if-unexp)
-		  (iter state stack #f #f)
-		  (parse-error state laval)))
-	     ((negative? stx)		; reduce
-	      (let* ((gx (abs stx))
-		     (gl (vector-ref len-v gx))
-		     ($$ (apply (vector-ref xct-v gx) stack)))
-		(iter (list-tail state gl)
-		      (list-tail stack gl)
-		      (cons (vector-ref rto-v gx) $$)
-		      lval)))
-	     ((positive? stx)		; shift
-	      (iter (cons stx state) (cons sval stack) #f (if nval lval #f)))
-	     (else			; accept
-	      (car stack))))))))))
-
-(define* (make-lalr-ia-parser mach #:key (skip-if-unexp '()))
-  (let* ((mtab (assq-ref mach 'mtab))
-	 (siu (map (lambda (n) (assoc-ref mtab n)) skip-if-unexp)))
-    (if (number? (caar (vector-ref (assq-ref mach 'pat-v) 0)))
-	;; hashed:
-	(make-lalr-ia-parser/num mach #:skip-if-unexp siu)
-	;; not hashed:
-	(make-lalr-ia-parser/sym mach))))
-
-;; @end itemize
 ;;; --- last line ---
